# ===================================================
# ===================================================
# Code to reproduce Tables 1-4 in simulations
# ===================================================
# ===================================================



library(snowfall)     # Parallel computing

sfInit(parallel = TRUE, cpus = 2, type = "SOCK")
#sfInit(parallel = FALSE)   # useful for trouble-shooting
sfLibrary(fGarch)
sfLibrary(sn)
sfLibrary(zoo)
sfLibrary(actuar)
source("RfnsSim.R")    # file contains the functions "CI.95.optim.k", "coverage.optim.k", "garchsim_alt", "APARCHX.sim", "G", "Expectile.est"
sfExport("CI.95.optim.k", "coverage.optim.k", "garchsim_alt", "APARCHX.sim", "G", "Expectile.est")


# ===========================================================
# Wrapper function for parallelization
# ===========================================================

# Inputs: 
# d: specifies the DGP
# alpha: confidence level of risk measures
# n: sample size
# estimation: If "TRUE" ("FALSE"), GARCH parameters are (not) estimated


wrapper <- function(d, alpha, n, estimation){
  CI.95.optim.k(d, AnzSim = 10000, alpha, n, estimation, cv.NA = 1.96, v = 10)
}

start <- Sys.time()

# ===================================================
# Table 1: n = 500
# ===================================================

res.500      <- sfLapply(c(1, 2), wrapper, alpha = c(0.95, 0.99, 0.995), n = 500, estimation = FALSE)
res.500.EST  <- sfLapply(c(1, 2), wrapper, alpha = c(0.95, 0.99, 0.995), n = 500, estimation = TRUE)

# EST = NO
res.500[[1]]$results.H    # rho = -4
res.500[[1]]$results.NP

res.500[[2]]$results.H    # rho = -1
res.500[[2]]$results.NP

# EST = YES
res.500.EST[[1]]$results.H    # rho = -4
res.500.EST[[1]]$results.NP

res.500.EST[[2]]$results.H    # rho = -1
res.500.EST[[2]]$results.NP



# ===================================================
# Table 2: n = 1000
# ===================================================

res.1000      <- sfLapply(c(1, 2), wrapper, alpha = c(0.99, 0.995, 0.999),  n = 1000, estimation = FALSE)
res.1000.EST  <- sfLapply(c(1, 2), wrapper, alpha = c(0.99, 0.995, 0.999),  n = 1000, estimation = TRUE)

# EST = NO
res.1000[[1]]$results.H    # rho = -4
res.1000[[1]]$results.NP

res.1000[[2]]$results.H    # rho = -1
res.1000[[2]]$results.NP

# EST = YES
res.1000.EST[[1]]$results.H    # rho = -4
res.1000.EST[[1]]$results.NP

res.1000.EST[[2]]$results.H    # rho = -1
res.1000.EST[[2]]$results.NP



# ===================================================
# Table 3: n = 2000
# ===================================================

res.2000      <- sfLapply(c(1, 2), wrapper, alpha = c(0.995, 0.999, 0.9995),  n = 2000, estimation = FALSE)
res.2000.EST  <- sfLapply(c(1, 2), wrapper, alpha = c(0.995, 0.999, 0.9995),  n = 2000, estimation = TRUE)

# EST = NO
res.2000[[1]]$results.H    # rho = -4
res.2000[[1]]$results.NP

res.2000[[2]]$results.H    # rho = -1
res.2000[[2]]$results.NP

# EST = YES
res.2000.EST[[1]]$results.H    # rho = -4
res.2000.EST[[1]]$results.NP

res.2000.EST[[2]]$results.H    # rho = -1
res.2000.EST[[2]]$results.NP



# ===================================================
# Table 4
# ===================================================

res.500.MIS  <- sfLapply(c(3, 4), wrapper, alpha = c(0.95, 0.99, 0.995),  n = 500, estimation = TRUE)

res.500.MIS[[1]]$results.H    # rho = -4
res.500.MIS[[2]]$results.H    # rho = -1


res.1000.MIS  <- sfLapply(c(3, 4), wrapper, alpha = c(0.99, 0.995, 0.999),  n = 1000, estimation = TRUE)

res.1000.MIS[[1]]$results.H    # rho = -4
res.1000.MIS[[2]]$results.H    # rho = -1


res.2000.MIS  <- sfLapply(c(3, 4), wrapper, alpha = c(0.995, 0.999, 0.9995),  n = 2000, estimation = TRUE)

res.2000.MIS[[1]]$results.H    # rho = -4
res.2000.MIS[[2]]$results.H    # rho = -1

print(Sys.time() - start)

sfStop()

